using System;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;

namespace Docsultant.Samples.Net.Common
{

	/// <summary>
	/// TcpListenerThread - represents thread that 
	/// manages listening server socket.
	/// </summary>
	public abstract class TcpListenerThread
	{
		TcpListener m_listener;
		Thread      m_thread;

		public TcpListenerThread(int nPort)
		{
			m_listener = new TcpListener(nPort);
		}
	
		public TcpListenerThread(String sLocalHost, int nPort)
		{
			IPAddress ip = Dns.Resolve(sLocalHost).AddressList[0];
			m_listener = new TcpListener(ip, nPort);
		}

		public void Start()
		{
			m_thread = new Thread(new ThreadStart(this.ThreadProc));
			m_thread.Start();
		}

		private void ThreadProc()
		{
			m_listener.Start();
			try
			{
				while(true)
				{
					TcpClient tcp = m_listener.AcceptTcpClient();
					if( !OnAccept(tcp) )
						break;
				}
			} catch(Exception e)
			{
				Debug.WriteLine(e, "TcpListenerThread");
			}
			m_listener.Stop();
		}

		public abstract bool OnAccept(TcpClient tcp);
	}

}
